/*
Written by Ed Hong
UWT
Jan. 22, 2003
modified June 8, 2003
modified Oct 4, 2003
modified Oct 1, 2004

 */

import java.util.*;

/**
 *  This class is a simple implementation of a priority queue
 *   priorities are integers
 *   lower numbered priorities are "higher priority";
 *   this means removeMin removes the item with the lowest priority number.
 * @author edhong
 * @version 0.0
 *
 */
public class SimplePQ {

  private ArrayList pqdata; // store all PQ items in an ArrayList

    /**
     * Constructor
     */
  public SimplePQ() {
	pqdata = new ArrayList();
  }


/** public SimplePQ.Locator class
  *   this class keeps track of an item inside the priority queue
  *   as it (possibly) moves around.
  *   If you use a locator to access an item that has already
  *     been removed from the priority queue, an error message is
  *     printed. (I did not bother to implement exceptions, which
  *         would be the more correct way of handling this)
  *
  */
  public class Locator {

	private int index; // current location of item inside pqdata.
	// if index==-1, then item is not inside pqdata.

    /** constructor..
     */
	private Locator() {
	    index = -1;
	}

	/**
	 *returns key (priority) associated to referenced data
         * prints error message if locator refers to item not in pq.
         */
    public int key() {
	    PQItem temp;
	    if (index ==-1) {
	 	    System.out.println("Locator error: item no longer in priority queue");
		    return -1;
	    }
	    temp = (PQItem) pqdata.get(index);
	    return temp.key;
	}

	/** returns Object associated to referenced data
         * prints error message if locator refers to item not in pq.
	 */
	public Object element() {
	      PQItem temp;
	      if (index ==-1) {
		  System.out.println("Locator error: item no longer in priority queue");
		  return null;
	      }
	      temp = (PQItem) pqdata.get(index);
	      return temp.element;
    }
	/** returns true if locator points to item still in pq,
         *  false otherwise
         */
	public boolean isvalid() {
	    return (index != -1);
	}
  }


    // internal class containing (priority, object,locator)
    // used to store PQ elements along with its priority and locator.
    // no methods beside constructor.
  private class PQItem {
     public int key;
     public Object element;
     public Locator loc;
     public PQItem(int k, Object o, Locator l) {
	    key = k;
	    element = o;
        loc = l;
     }
  }

    /** insert item o with priority key into pq */
  public void insertItem(int key, Object o) {
	insert(key,o);
  }

    /** insert item o with priority key into pq; return locator */
  public Locator insert(int key, Object o) {
     Locator l = new Locator();
     PQItem temp;
	 temp = new PQItem(key, o, l);
	 l.index = pqdata.size(); // set position of new  node.
	 pqdata.add(temp);  // add temp to end of ArrayList (at position l.index)
	 return l;
  }

  /** returns the index of the item with smallest priority in pqdata, or -1 if there's
      no data in pqdata */
  private int findMinIndex() {
	ListIterator pqdatiter;
	int index; // stores index of element with smallest key
	PQItem temp; // stores current PQItem being looked at.
	int minkey;  // stores smallest key

	pqdatiter = pqdata.listIterator();
	if (!pqdatiter.hasNext()) return -1;
	temp = (PQItem) pqdatiter.next();
	minkey = temp.key;
	index = 0;

	while (pqdatiter.hasNext()) {
	    temp = (PQItem) pqdatiter.next();
	    if (temp.key < minkey) {
			minkey = temp.key;
			index = pqdatiter.previousIndex();
	    }
	}
	return index;
  }

    /** removes and returns item o with highest priority (smallest
        priority number). */
  public Object removeMin() {
	int ind; // stores index of element with smallest key
	PQItem temp, retval;

	if (pqdata.size()==0) return null;

	/*	// Debugging status statements
	for (int i = 0; i < pqdata.size(); i++) {
	    temp = (PQItem) pqdata.get(i);
	    System.out.println("PQDAT: i " + i + " temp " + temp + " " +
			       temp.loc.index);
			       } */

	// search for item with smallest key.
	ind = findMinIndex();

	retval = (PQItem) pqdata.get(ind);
	retval.loc.index = -1;

	// now move items after position ind down by one, updating
	// locators as we go along.
	for (int i = ind+1; i < pqdata.size(); i++) {
        temp = (PQItem) pqdata.get(i);
	    pqdata.set(i-1, temp);
	    temp.loc.index = i-1;
	}

	pqdata.remove(pqdata.size()-1); // remove last item in arraylist.

	return retval.element;
  }

    /** returns item o with highest priority (smallest
        priority number). Does not remove item from priority queue.*/
  public Object minElement() {
	int ind; // stores index of element with smallest key
	PQItem retval;

	if (pqdata.size()==0) return null;

	// search for item with smallest key.
	ind = findMinIndex();

	retval = (PQItem) pqdata.get(ind);
	return retval.element;
  }

  public void replaceKey(Locator l, int newkey) {
	System.out.println("Error: replaceKey not implemented yet!");
  }

  public void remove(Locator l) {
	System.out.println("Error: remove not implemented yet!");
  }

  public boolean isEmpty() {
	return (pqdata.size() == 0);
  }

    /** this is used only for debugging..*/
  public static void main (String args[]) {
	SimplePQ.Locator l1, l2;
	SimplePQ Q;
	Q = new SimplePQ();
        l1=Q.insert(1,"item1");
        System.out.println("l1 is " + l1);
        System.out.println("l1.key() is " + l1.key());
        System.out.println("l1.element() is " + l1.element());
        System.out.println("l1.isvalid() is " + l1.isvalid());
        l2=Q.insert(2,"item2");
        System.out.println("l2 is " + l2);
        System.out.println("l2.key() is " + l2.key());
        System.out.println("l2.element() is " + l2.element());
        System.out.println("l2.isvalid() is " + l2.isvalid());
	Q.removeMin();
        System.out.println("removeMin called");
        System.out.println("l1 is " + l1);
        System.out.println("l1.key() is " + l1.key());
        System.out.println("l1.element() is " + l1.element());
        System.out.println("l1.isvalid() is " + l1.isvalid());
        System.out.println("l2 is " + l2);
        System.out.println("l2.key() is " + l2.key());
        System.out.println("l2.element() is " + l2.element());
        System.out.println("l2.isvalid() is " + l2.isvalid());
	Q.removeMin();
        System.out.println("removeMin called");
        System.out.println("l1 is " + l1);
        System.out.println("l1.key() is " + l1.key());
        System.out.println("l1.element() is " + l1.element());
        System.out.println("l1.isvalid() is " + l1.isvalid());
        System.out.println("l2 is " + l2);
        System.out.println("l2.key() is " + l2.key());
        System.out.println("l2.element() is " + l2.element());
        System.out.println("l2.isvalid() is " + l2.isvalid());
  }
}

