/*
Written by Ed Hong
UWT
Oct. 5, 2003
Modified Oct. 1, 2004.
 */

import java.util.*;

/**
 *  This class is a simple implementation of the Task organizer
 *  program. It illustrates use of a locater-based priority queue.
 * @author edhong
 * @version 0.1
 *
 */
public class TaskOrganizer{

    /** priority queue storing tasks that have not yet been done */
  private static SimplePQ taskdata;

    /** array of locators that point to tasks in taskdata
     * task with task ID i will be stored in allLocators at index i.
     */
  private static ArrayList allLocators;

    /** task ID of the next task entered. Increases by one with each task
        inserted. nexttaskID never decreases, even when a task is finished. */
  private static int nexttaskID = 0;

    /** use Scanner.java for parsing input */
  private static Scanner keyboard;

    /** main program */
  public static void main (String args[]) {
	// initialize things..
	taskdata = new SimplePQ();
	allLocators = new ArrayList();
	keyboard = new Scanner(System.in);

	// add some initial tasks, so we don't have to type them in
        // all the time.
	addtask(2, "Finish TCSS343 Homework.");
	addtask(1, "Shop for food.");
	addtask(10, "Schedule Dentist appointment");
	addtask(5, "Brush teeth");
	addtask(8, "Call George");
	addtask(5, "Clean Room.");
	addtask(2, "Drink milk");
	System.out.println("");

	print_full_directions();
	System.out.println("");
	viewtask();

	// main processing loop
	String command;
	String taskname;
	int priority;
	while (true) { // repeat forever
		System.out.println("");
	    print_command_prompt();
	    command = keyboard.nextLine();
	    if (command.equals ("a")) {
			System.out.println("Adding task: Enter task priority:");
			priority = keyboard.nextInt();
			keyboard.nextLine();
			System.out.println("Adding task: Enter task name:");
			taskname = keyboard.nextLine();
			addtask(priority,taskname);
		}
	    else if (command.equals("l")) {
			System.out.println("Listing all tasks");
			listtasks();
	    }
	    else if (command.equals("p")) {
			System.out.println("Performing next task");
			performtask();
	    }
	    else if (command.equals("q")) {
			System.out.println("Quitting program..");
			return;
	    }
	    else if (command.equals("v")) {
		    viewtask();
	    }
	    else {
			System.out.println("Error: command not understood.");
		    viewtask();
	    }
	}
  }

    /** prints out all directions */
  public static void print_full_directions() {
	System.out.println("This is the Task Organizer program.");
	System.out.println("You can use this to help you manage your time.");
	System.out.println("It will keep track of a set of tasks that you enter.");
	System.out.println("You will assign each task an integer priority.");
	System.out.println("Lower priority numbers represent more important tasks.");
	System.out.println("Each task will be automatically given a unique task ID  that never changes.");
	System.out.println("This program will tell you which task to do next based on the priorities.");
	System.out.println("At the command prompt, you will be able to do the following:");
    System.out.println("(a)dd a new task. You will be prompted to enter the task name and priority.");
	System.out.println("(l)ist all tasks. This shows all active tasks.");
	System.out.println("(p)erform next task. This indicates that you have completed the next task that ");
	System.out.println("   is most important.");
	System.out.println("(r)emove a task. You will be prompted to enter the task number.");
	System.out.println("(c)hange the priority of a task. You will be prompted to enter the task number");
	System.out.println("          and the new priority.");
	System.out.println("(v)iew the next task that needs to be done.");
	System.out.println("(q)uit the program");
  }

    /** adds new task to priority queue.
     * @param priority Priority of new task
     * @param name  Name of new task
     * stores locator in allLocators array.
     *  prints out new task information (including task #).
     */
  public static void addtask(int priority, String name) {
	SimplePQ.Locator ll;
	int newtaskID;
	ll = taskdata.insert(priority, name);
	newtaskID = nexttaskID;
	allLocators.add(ll); // add locator to the end of the arraylist
	nexttaskID++;
	System.out.println("  Added the following new task:");
	System.out.println("  Task ID:" + newtaskID
			   + ", prior: " + priority
			   + ", name: " + name);
  }

    /** prints out short list of possible commands */
  public static void print_command_prompt() {
	System.out.println("Enter command: (a)dd (l)ist (p)erform (r)emove (c)hange (v)iew (q)uit");
  }

    /** lists all tasks that need to be done */
  public static void listtasks() {
		int j;
		ListIterator locIter;

		locIter = allLocators.listIterator();
		while (locIter.hasNext()) {
		    SimplePQ.Locator loc;
			loc = (SimplePQ.Locator) locIter.next();
			if (loc.isvalid()) {
			 // only tasks with valid locators are active (still need to be performed)
				System.out.println("  Task ID:" + locIter.previousIndex()
								   + ", prior: " + loc.key()
								   + ", name: " + loc.element());
			}

		}
  }

    /** prints out next highest priority task (task with lowest priority number) */
  public static void viewtask() {
	if (taskdata.isEmpty()) {
	    System.out.println(  "No tasks to be done. Go take a break");
	}
	else {
	    System.out.println("The next highest priority (lowest priority number) task is:");
	    System.out.println("  " + taskdata.minElement());
	}
  }

    /** prints out next highest priority task (task with lowest priority
     *   number). Removes this task from the queue, marking it as done. */
  public static void performtask() {
	String tname = (String) taskdata.removeMin();
	System.out.println("  You have just completed task: " + tname);
  }
}



